;animated objects routines are placed here
		
CheckBackAnimations
;check if bat is in current room		
	LD A,(BatStatus)
	OR A
	CALL NZ, DisplayBat	
	
; check if rotating fan is in current room		
	LD A,(FanStatus)
	OR A
	CALL NZ, DisplayFan	
	
	LD A,(BadWireRoomStatus)
	OR A
	CALL NZ,DisplayWire
	
	LD A,(SafeFireStatus)
	OR A
	CALL NZ,DisplaySafeFire
	
	RET

;----------------------
CheckFrontAnimations
;check if fire is in current room
	LD A,(FireStatus)
	OR A
	CALL NZ, DisplayFire
	
;check if calling for help is in this room
	LD A,(CallHelpRoomStatus)
	OR A
	CALL NZ,DisplayCallHelp	
	
;check if any tiles in front of player
	LD A,(FrontTileStatus)
	OR A
	CALL NZ, DrawFrontTile	
	
;check if player is in swimming pool room
	LD A,(AcidStatus)
	OR A
	CALL NZ,DisplayAcid		
	
;check if zombie is in this room
	LD A,(ZombieStatus)
	OR A
	CALL NZ,DisplayZombie		
	RET	

		
;--------FLAMES-------------		
;display burning flames	
DisplayFire
	;count fire frame		
	LD A,(GameCycle)
	RRA ; cycle 1-8 gives 1-4 but slower
	LD B,A
	INC B
	LD DE,24
	LD HL,BigFire1-24
CountFireFrame	
	ADD HL,DE
	DJNZ CountFireFrame		
	EX DE,HL
		
;Drawing fire		
	;Read Fire Data
	PUSH DE
	LD DE,FirePos
	LD A,(DE)
	LD L,A
	LD H,80;position
	INC DE;go fo fire size
	LD DE,FireSize
	LD A,(DE)
	LD B,A
	LD C,1
	POP DE
	CALL TileBlock;draw fire
	RET
	
;------CALL FOR HELP-------
DisplayCallHelp
	;is the girl still calling for help?
	LD A,(CallHelpActive)
	OR A
	RET NZ; return if she isn't
	
	LD DE,CallHelpPic
	LD H,72
	LD A,(CallHelpL)
	LD L,A
	LD A,(GameCycle)
	
	BIT 2,A; only IN some cycles display CALL for help
	JR Z,NotDisplayCallHelp 	
	CALL DrawTile
	RET
NotDisplayCallHelp
	HALT ;to make animation smoother	
	LD BC,770;B=3,C=2
	PUSH BC
	LD DE,EmptySpace
	CALL TileBlock
	POP BC
	LD D,89
	LD A,(CallHelpL)
	LD E,A
	LD A,71
	CALL AttrBlock
	RET
	
;--------------------
;draw box in front of player
DrawFrontTile	
	LD HL,FrontTileNr
	LD A,(HL)
	CALL CountTileAddr; tile addr in DE
	INC HL
	LD L,(HL) ;position
	LD H,80
	CALL DrawTile
	
	
	RET			
;-------------BAT--------------------			
DisplayBat
;the bat goes up and down
	LD HL,CurrentBatCycle
	LD A,(HL)
	OR A 				; 0 means reaching movement limit and going back
	CALL Z, ChangeBatDirection

;check if bat goes up or down	
						; HL=current bat cycles no matter if routine was called or not
	INC HL 				; HL=BatUpOrDown
	LD A,(HL)
	OR A
	PUSH AF
	CALL NZ,BatDown
	POP AF
	CALL Z,BatUp
	
;decrease bat cycle	left to turn back
	LD HL,CurrentBatCycle
	DEC (HL)
	
;check collision with player
	LD DE,(NewPlayerPos)
	LD HL,(BatPosL)	


	LD A,L				;compare collumns - L
	AND 00011111B
	LD L,A
	LD A,E
	AND 00011111B
	SUB 3				
	CP L				;player too far left of bat?
	RET NC				
	ADD A,6
	CP L				;player too far right of bat?
	RET C
	
	LD DE,(NewPlayerPos);collumns confirmed
	LD HL,(BatPosL)					
	
	CALL HLtoLine
	LD B,A			;bat top line in B
	
	EX DE,HL
	CALL HLtoLine
	LD C,A			;player top line in C
		
					
	LD A,B
	ADD A,16
	CP C
	RET C			;bat too high for collision
	
	LD A,C
	ADD A,48
	CP B
	RET C			;bat too low for collision
	
;collision!
	XOR A
	OUT (254),A
	LD HL,PlayerEnergy
	DEC (HL)		;lose energy
	
	LD A,(HL)
	OR A
	JP Z,GameOver	;0=all energy lost
	
	LD HL, 16449	;draw new energy bar
	CALL DrawBar
	
	LD A,16
	OUT (254),A
	
	RET

;-------			
ChangeBatDirection
	INC HL			; HL=BatUpOrDown
	LD A,(HL)
	CPL 			; 00000000 becomes 11111111 and vice versa
	LD (HL),A		; Direction changed
	
	DEC HL
	DEC HL			; HL=bat cycles
	LD A,(HL)
	INC HL			; N bat cycles means that a bat will go N pixel up or down before it turns back
	LD (HL),A		; current bat cycle=bat cycles
	RET		
				
;--------Animating bat---------
BatDown		
;bat going down	

	;get bat coordinates into HL
	LD HL,(BatPosL)

	CALL OnePixelLineDown 	;go down
	JR DrawBat
	
;----------
BatUp	
	;get bat coordinates into HL
	LD HL,(BatPosL)

	CALL OnePixelLineUp	;go up
	
		
;-------------		
DrawBat
;HL already contains screen position of bat

	LD (BatPosL),HL	;store new position of bat
	LD BC,784 ;size (B=3,C=16)

;bat has 2 frames of animation, using game cycles choose which one should be used now
	LD DE,Bat01
	LD A,(GameCycle)
	BIT 2,A
	JR NZ, DrawBat2
	LD DE,Bat02

DrawBat2
	;draw bat at new position, it clears bat at old position
	PUSH BC
	PUSH HL
DrawBat3
	LD A,(DE)		;read bat byte
	LD (HL),A		;and write it to screen
		
	INC L			;next collumn
	INC DE			;next bat byte
	DJNZ DrawBat3
	POP HL
	POP BC

	CALL OnePixelLineDown;go to next line	

	DEC C ;decrease number of lines left to do
	JR NZ,DrawBat2
	RET	
;-------------
HLtoLine
;take HL coordinates of byte and change it into
;horizontal line 0-192
;HL: coordinates
	PUSH BC
	LD A,H
	AND 00000111B
	LD B,A
	
	LD A,H
	RLCA
	RLCA
	RLCA
	AND 11000000B
	LD C,A
	
	LD A,L
	AND 11100000B
	RRCA
	RRCA	
	
	OR B
	OR C
	POP BC
	RET
	
	

;-----------FAN------------
DisplayFan
	;position IN HL
	LD HL,FanH
	LD D,(HL)
	INC HL
	LD E,(HL)
	EX DE,HL 		; position
	
;count frame in DE	
	PUSH HL
	LD HL, Fan01-38
	LD DE,38
	LD A,(GameCycle)
	RRA 			; 0-7 -> 0-3
	INC A
CountFanFrame	
	ADD HL,DE
	DEC A
	JR NZ,CountFanFrame
	
	EX DE,HL		; frame sprite in DE
	POP HL			; position in HL	
	CALL DrawTile
	RET
		

;-----------WIRE------------
DisplayWire
	LD HL,256*80+88
	LD A,(BadWireStatus)
	CP 2					;is bad wire fixed?
	JR Z,DisplayWire3		;status=2 - wire repaired

;sparkling electricity animation
	LD DE,EmptySpace2
	LD A,(GameCycle)
	BIT 2,A					;1st frame or 2nd frame?
	JR Z,DisplayWire2
	
	LD DE,Electricity
		
DisplayWire2
	CALL DrawTile
	RET						
	
DisplayWire3
	LD 	H,84				;wire is repaired
	LD A,255
	LD (HL),A
	INC H
	LD (HL),A
	RET

;---	
Electricity
	.BYTE 1,1
	.BYTE	  0,  0, 34,119,221,136,  8,  0
	.BYTE 70
	
;-----------SAFE FIRE------------

DisplaySafeFire	
	;safe fire animation
	LD HL,80*256+86
	LD BC,2*256+1
	LD DE,LiftFlame1
	LD A,(GameCycle)
	BIT 2,A
	JR Z,DisplaySafeFire2

	LD DE,LiftFlame2
	
DisplaySafeFire2		
	CALL TileBlock
	RET
	
;-------ACID--------------
DisplayAcid
	LD A,(GameCycle)
	AND 00000110B		;tile cycling
	RRA
	OR A
	JR Z,DisplayAcid1
	CP 1
	JR Z,DisplayAcid2
	CP 2
	JR Z,DisplayAcid3		

	LD DE,Acid4
	JR DisplayAcid4
	
DisplayAcid1
	LD DE,Acid1
	JR DisplayAcid4
DisplayAcid2
	LD DE,Acid2
	JR DisplayAcid4
DisplayAcid3
	LD DE,Acid3
	
	
DisplayAcid4
	LD BC,9*256+1
	LD HL,80*256+168
	CALL TileBlock
;check if player is protected from acid
	LD HL,(NewPlayerPos)

;check player position		
	LD A,H			;player in swimming pool: H=80, l>32
	CP 80	
	RET NZ
	
	LD A,L
	CP 33
	RET C

;player in pool confirmed now
	LD A,(ChosenItem)
	CP 10				;rubber boots?	
	JR NZ,DisplayAcidDeath
;player wearing boots
	LD HL,BootsPoolStatus	;message about acid already seen?
	LD A,(HL)
	OR A
	RET NZ	
;type message about boots which protected from acid
	INC (HL)			;change pool status, don't display this message again
	CALL IncreaseScore
	LD A,71
	CALL TypeMessage
	RET
	
DisplayAcidDeath
	LD A,70
	CALL TypeMessage
	JP GameOver
	

;-----------
Acid1
	.BYTE 2,2
	.BYTE	  0, 56,124,254,255,255,255,254
	.BYTE	  4, 14, 31,127,253,255,127, 63
	.BYTE	247,255,255,191, 31,191,255,255
	.BYTE	127,247,227,247,255,255,255,223
	.BYTE	  4,  4,  4,  4
	

Acid2
	.BYTE 2,2
	.BYTE	  0,  0, 56,254,255,255,247,255
	.BYTE	  0,  0, 30,127,255,255,255,247
	.BYTE	255,191, 31,191,255,255,254,255
	.BYTE	227,247,255,255,223,127, 63,125
	.BYTE	  4,  4,  4,  4

Acid3
	.BYTE 2,2
	.BYTE	  0,  0,195,231,255,255,255,191
	.BYTE	  0,  0,193,243,255,247,227,247
	.BYTE	 31,191,255,254,255,255,255,255
	.BYTE	255,223,127, 63,125,255,255,255
	.BYTE	  4,  4,  4,  4
	
Acid4
	.BYTE 2,2
	.BYTE	  1,131,199,255,255,191, 31,191
	.BYTE	  0,129,195,255,255,223,255,255
	.BYTE	255,254,255,247,255,255,255,255
	.BYTE	125, 63,127,255,255,247,227,247
	.BYTE	  4,  4,  4,  4
	
	
;------------ZOMBIE--------------
DisplayZombie
	LD A,(ZombieDirection)
	CP 255						;zombie walking right?
	JR Z,DisplayZombieRight
	

;-----------
;zombie walk left
DisplayZombieLeft
	LD A,(GameCycle)
	OR A
	JR Z,ZombieWalkLeft		;game cycle=0 -> move zombie
	
	BIT 2,A
	JR NZ,DisplayZombieLeft2	;sprite frame
	
DisplayZombieLeft1
	LD DE,ZombieFrame1L
	JR DisplayZombieEnd
	
DisplayZombieLeft2
	LD DE,ZombieFrame2L
	JR DisplayZombieEnd
	
;------	
ZombieWalkLeft
;clear zombie right side	
	LD H,80
	LD A,(ZombiePosition)
	INC A
	LD L,A
	LD DE,EmptySpace2
	LD BC,1*256+6
	CALL TileBlock
;restore floor attribute
	LD H,90
	LD L,192
	LD B,(HL)				;get floor attribute from screen (left floor edge)	
	LD A,(ZombiePosition)
	INC A
	ADD A,160				;position of zombie leg
	LD L,A
	LD (HL),B
	
;change zombie position
	LD HL,ZombiePosition
	DEC (HL)
;check if zombie reached edge of movement
	LD B,(HL)				;position
	LD A,(ZombieLeftEdge)
	CP B					;compare position to edge	
	JR	NZ,DisplayZombieLeft1	;edge not reached
	
;change direction
	LD A,(ZombieDirection)
	NEG
	LD (ZombieDirection),A
	JR	DisplayZombieLeft1
	
	
;-----------
;zombie walk right	
DisplayZombieRight
	LD A,(GameCycle)
	OR A
	JR Z,ZombieWalkRight		;game cycle=0 -> move zombie
	
	BIT 2,A
	JR NZ,DisplayZombieRight2	;sprite frame
	
DisplayZombieRight1
	LD DE,ZombieFrame1R
	JR DisplayZombieEnd
	
DisplayZombieRight2
	LD DE,ZombieFrame2R
	JR DisplayZombieEnd
	
;------	
ZombieWalkRight
;clear zombie left side	
	LD H,80
	LD A,(ZombiePosition)
	LD L,A
	LD DE,EmptySpace2
	LD BC,1*256+6
	CALL TileBlock
;restore floor attribute
	LD H,90
	LD L,192
	LD B,(HL)				;get floor attribute from screen (left floor edge)	
	LD A,(ZombiePosition)
	ADD A,160				;position of zombie leg
	LD L,A
	LD (HL),B
	
;change zombie position
	LD HL,ZombiePosition
	INC (HL)
;check if zombie reached edge of movement
	LD B,(HL)				;position
	LD A,(ZombieRightEdge)
	CP B					;compare position to edge	
	JR	NZ,DisplayZombieRight1	;edge not reached
	
;change direction
	LD A,(ZombieDirection)
	NEG
	LD (ZombieDirection),A
	JR	DisplayZombieRight1
	
;---------	
DisplayZombieEnd
;displayZombie	
	LD H,80
	LD A,(ZombiePosition)		;read position
	LD L,A
	CALL DrawTile

;check if zombie gets player	
	;RET							;TEMP!!!
	LD A,(ZombiePosition)
	AND 00011111B				;extract collumn
	SUB 4						
	LD B,A						;left edge of dangerous area
	ADD A,6
	LD C,A						;right edge of dangerous area
	LD A,(NewPlayerPos)		;get player L
	AND 00011111B				;extract collumn
	CP B
	RET C						;safe, far too left from zombie
	CP C
	RET NC						;safe, far too right from zombie
	
;otherwise,player is dead	
	LD A,55
	CALL TypeMessage			;death from zombie message
	CALL GameOver
	RET

	
;--------------
ZombieFrame1L
	.BYTE 2,6
	.BYTE	  0,  0,  1,  3,  3,  4,  4, 15
	.BYTE	  0,240,248,248,252,108,244,228
	.BYTE	  0,  5,  0,  3,  1,  1,  2,  5
	.BYTE	204,120,248,240,224, 48,216,168
	.BYTE	  5,  5,  5, 14,  6, 10,  7, 11
	.BYTE	204,172,212,228,212,228,116,120
	.BYTE	  7,  3,  7,  3,  3,  3,  2,  6
	.BYTE	 56,184,180, 52,118,118,114,250
	.BYTE	  6, 14, 15, 15, 15, 31, 31, 30
	.BYTE	170,168,  4,124, 60, 56, 56, 60
	.BYTE	 14, 14, 14, 14, 30, 30, 60,124
	.BYTE	 30, 30, 15, 15,  7, 14, 62,124
	.BYTE	 68, 68, 68, 68, 68, 68, 68, 68
	.BYTE	 68, 68, 84, 84
	
ZombieFrame2L
	.BYTE 2,6
	.BYTE	  0, 15, 31, 63, 63, 70, 79,254
	.BYTE	  0,  0,128,128,192,192, 64, 64
	.BYTE	 12, 87, 15, 63, 30, 19, 45, 90
	.BYTE	192,128,128,  0,  0,  0,128,128
	.BYTE	 92, 90, 93,238,109,174,119,183
	.BYTE	192,192, 64, 64, 64, 64, 64, 64
	.BYTE	119, 55,119, 38, 46, 46, 14, 95
	.BYTE	 64, 64, 64,224,224,224, 96, 96
	.BYTE	 85,213,224,247,243,243,243,243
	.BYTE	 96, 64,128,192,192,128,128,128
	.BYTE	123,123,123, 61, 29, 57,119,239
	.BYTE	128,128,128,192,192,192,192,128
	.BYTE	 68, 68, 68, 68, 68, 68, 68, 68
	.BYTE	 68, 68, 84, 84



ZombieFrame1R
	.BYTE 2,6
	.BYTE	  0, 15, 31, 31, 63, 54, 47, 39
	.BYTE	  0,  0,128,192,192, 32, 32,240
	.BYTE	 51, 30, 31, 15,  7, 12, 27, 21
	.BYTE	  0,160,  0,192,128,128, 64,160
	.BYTE	 51, 53, 43, 39, 43, 39, 46, 30
	.BYTE	160,160,160,112, 96, 80,224,208
	.BYTE	 28, 29, 45, 44,110,110, 78, 95
	.BYTE	224,192,224,192,192,192, 64, 96
	.BYTE	 85, 21, 32, 62, 60, 28, 28, 60
	.BYTE	 96,112,240,240,240,248,248,120
	.BYTE	120,120,240,240,224,112,124, 62
	.BYTE	112,112,112,112,120,120, 60, 62
	.BYTE	 68, 68, 68, 68, 68, 68, 68, 68
	.BYTE	 68, 68, 84, 84
	
ZombieFrame2R
	.BYTE 2,6
	.BYTE	  0,  0,  1,  1,  3,  3,  2,  2
	.BYTE	  0,240,248,252,252, 98,242,127
	.BYTE	  3,  1,  1,  0,  0,  0,  1,  1
	.BYTE	 48,234,240,252,120,200,180, 90
	.BYTE	  3,  3,  2,  2,  2,  2,  2,  2
	.BYTE	 58, 90,186,119,182,117,238,237
	.BYTE	  2,  2,  2,  7,  7,  7,  6,  6
	.BYTE	238,236,238,100,116,116,112,250
	.BYTE	  6,  2,  1,  3,  3,  1,  1,  1
	.BYTE	170,171,  7,239,207,207,207,207
	.BYTE	  1,  1,  1,  3,  3,  3,  3,  1
	.BYTE	222,222,222,188,184,156,238,247
	.BYTE	 68, 68, 68, 68, 68, 68, 68, 68
	.BYTE	 68, 68, 84, 84
	
	
	
	