DrawRoom
; draw a single game location on the screen
;HL: room data address	

;store start of current room data
	LD (CurrentRoom),HL		
		
;clear screen for new location	
	PUSH HL

;Clear animated objects data
	XOR A	
	LD (FireStatus),A
	LD (BatStatus),A
	LD (FanStatus),A
	LD (CallHelpRoomStatus),A
	LD (FrontTileStatus),A	
	LD (BadWireRoomStatus),A
	LD (SafeFireStatus),A
	LD (AcidStatus),A
	LD (ZombieStatus),A	

	CALL ClearRoom	
	
;Room title
;draw  stripe in the text line 
	LD HL,16544
	LD DE,Border01
	LD BC,8193;B=32,C=1
	CALL TileBlock
	
;print room title	
	POP HL			; restore room data address
	LD A,(HL)		; read position of title
	INC HL
	PUSH AF			; store position
	LD A,(HL)
	LD E,A
	INC HL
	LD A,(HL)	
	LD D,A			; DE loaded with text address	
		
	
	POP AF			; restore position
	PUSH HL			; store room data address
	LD H,64
	LD L,A 			; position of room title
	CALL TypeText
	POP HL			; restore room data address
	
	INC HL			; go to next room data
		
		

;1.Draw tile blocks	

;Each block is stored with 4 bytes
;Byte 1: number of tile
;Byte 2,3: size (B-width,C-height) of block 
;Byte 4:  L coordinate of left upper corner
	;(the whole address is stored IN HL, but H can have only 2 values (we use 2 screen segments), so to save memory
	;H isn't stored and tile drawing loop is done twice  with  H=72 and H=80		
		
		LD A,72
		LD (SecondOrThirdSegment),A			
		
RoomBlocks1
		LD A, (HL)				;number of tile	loaded in A
		OR A
		JR Z, RoomBlocks2		;if (HL)=0 that means either end of drawing blocks
								;or going to next of screen segments
								;(increasing H by 8)
		
		CALL CountTileAddr		;count address of tile in memory and puts it into DE
		
		INC HL 
		LD A,(HL)
		LD B,A
		INC HL
		LD A,(HL)
		LD C,A			;BC is loaded with block size
				
		INC HL
		PUSH HL			;HL containing  room data address stored
		
		LD A,(HL)			
				
		LD L,A			
		LD A, (SecondOrThirdSegment)	
		LD H,A			;HL is loaded with block left upper corner
		
		
		CALL TileBlock	;draw block of tiles
		
		POP HL			;restore room blocks data address 
		INC HL			;start of next block data
		JR RoomBlocks1
		
SecondOrThirdSegment	.BYTE 72	

RoomBlocks2
;go to another screen block or end  blocks drawing	

		INC HL							;go to next byte of room data
		LD A,(SecondOrThirdSegment)
		CP 80							;check if it is last screen segment
		JR Z, RoomTiles1 				;if so go to single tiles drawing
		ADD A,8							;if not go to another screen segment
		LD (SecondOrThirdSegment),A
		JR RoomBlocks1

		
;2. Draw single tiles

;Tiles are stored with 2 bytes
	;1st byte: number of tile
	;2nd byte: L of HL storing left upper tile corner (H is 72 or 80)
		
RoomTiles1
		LD A,72
		LD (SecondOrThirdSegment),A
		
RoomTiles2
		LD A, (HL)				;number of tile	loaded in A
		
		OR A
		JR Z, RoomTiles3		;if (HL)=0 that means either end of drawing blocks
								;or going to next of three screen segments
								;(increasing H by 8)
		
		CALL CountTileAddr		;count address of tile in memory and put it in DE

		INC HL
		PUSH HL					;store room data address
		LD A, (HL)
		LD L,A
		LD A, (SecondOrThirdSegment)
		LD H,A					;put upper left tile corner in HL
						
		CALL DrawTile			;call  routine drawing single tile
		
		POP HL					;restores room data address
		INC HL					;go to next room data byte
		JR RoomTiles2
					
RoomTiles3
		INC HL					;go to next byte of room data
		LD A,(SecondOrThirdSegment)
		CP 80					;check if last screen part
		JR Z, RoomAttrs1 		;if so end drawing tiles
		ADD A,8					;if not go to another screen segment
		LD (SecondOrThirdSegment),A
					
		JR RoomTiles2

		
;3. Attributes
		;Fills rectangular blocks with given attribute
		;Attribute blockss are stored with 4 bytes
			;1st byte: 			attribute
			;2nd and 3rd byte: 	left upper block corner address
			;4th and 5th byte: 	attr block size
					
RoomAttrs1 
		LD A,(HL)			;load attribute from memory
		OR A				;0 means end of writing attributes 		
		JR Z,RoomMirror1
		PUSH AF
		INC HL				;write screen address to DE
		LD A,(HL)
		LD D,A
		INC HL
		LD A,(HL)
		LD E,A
		INC HL				;write attributes block size to BC
		LD A,(HL)
		LD B,A				;width
		INC HL
		LD A,(HL)
		LD C,A				;height
		INC HL
		POP AF				;restore attribute
		CALL AttrBlock		;draw attributes
		
		JR RoomAttrs1

;4.Mirror some areas of screen		

;Mirrors are stored with 4 bytes:		
	;1st and 2nd byte:  position of mirrored area 
	;3rd and 4th byte:  size of mirrored area in characters
	
RoomMirror1		
	INC HL
	LD A,(HL)
	OR A			; 0=end of mirrored areas
	JR Z,DrawItemsInThisRoom
	LD D,A
	INC HL
	LD A,(HL)
	LD E,A			; position IN DE
	INC HL
	LD A,(HL)
	LD B,A
	INC HL
	LD A,(HL)
	LD C,A			; size in BC
	PUSH HL
	EX DE,HL		; position IN HL
	CALL MirrorArea ; make mirror
	POP HL		
	JR RoomMirror1	

	
;5. Items
DrawItemsInThisRoom
	NOP
	PUSH HL			; store room data address
	CALL DrawItems
	POP HL			; restore room data address
	

;Drawing room is finished
;but we must read some room parameters
	
		
; 6. Set neighbouring rooms		
RoomNeighbours
	INC HL
	LD E,(HL)	
	INC HL
	LD D,(HL)		
	LD (LeftEdgeRoom),DE 	; left neighbour stored 

	INC HL
	LD E,(HL)	
	INC HL
	LD D,(HL)		
	LD (RightEdgeRoom),DE 	; right neighbour stored

; 7. Room code	
;InitRoomCode contains address of code which is executed after drawing room
;this code is responsible for special effect in this room
	INC HL
	LD E,(HL)
	INC HL
	LD D,(HL)
	LD (InitRoomCode),DE; store room code

; 8. Events
;Current Events contain address of events data. Events happen after pressing fire
	INC HL
	LD (CurrentEvents),HL
		
; 9. Executing room code
	
	LD A,D
	OR E					; if room code=0, do not execute room code
	JR Z,RoomCodeEnd		
	EX DE,HL				; load room code address IN HL
	JP (HL)					;go to room code, execute it and after RET in room code subroutine end of drawing room
	
RoomCodeEnd		
	RET						;room drawing is finished
	
	
;------------------------------
CountTileAddr	
;exchange one byte tile number in A to tile address in DE
		;A-tile number
		;DE- tile address as routine result
		;BC is used	

	LD BC,TileAddrData
	PUSH AF
	ADD A,C
	LD C,A
	LD A,0
	ADC A,B
	LD B,A
	POP AF
	ADD A,C
	LD C,A
	LD A,0
	ADC A,B
	LD B,A	;position of sprite address IN memory placed IN BC	
	LD A,(BC)
	LD E,A
	INC BC
	LD A,(BC)
	LD D,A	;sprite address placed IN DE
	RET

	
;-----------------------------------------------------------------------------
TileBlock
;draw a block of tiles (one tile used)
	;BC: block size (B - width, C - height)
	;DE: tile address
	;HL: left upper corner of the block	

		
TileBlock1
		PUSH BC
		PUSH DE
		PUSH HL

;draw one horizontal line of tiles
TileBlock2		
		PUSH BC
		PUSH DE
		PUSH HL
		CALL DrawTile		; draw a single tile

		POP HL
		POP DE 				; restore screen position and tile address
	
		LD A,(DE)
		LD B,A 				; read tile width from memory and write to B
	
		LD A,L
		ADD A,B
		LD L,A				; count horizontal coordinate of next tile in the line
	 						;(add B=tile width to coordinate of previous tile)
			
		POP BC				;restore block size
					
		DJNZ TileBlock2		;repeat until all line of tiles is drawn
		
		
; draws C lines of tiles
		POP HL
		POP DE
	
		INC DE
		LD A,(DE)			;read tile height from memory
		LD C,A				;write tile height to C	
		DEC DE	
	
		LD B,L				;B is used to check carry		
		LD A,L	
TileBlock3	
		ADD A,32 			;go C lines down,because C=tile height
		DEC C
		JR NZ, TileBlock3
		LD L,A				
		
;check if new tile begins in next screen segment
;(carry happens when L<B(number exceeded 256)	
		SUB B	
		JR NC, TileBlock4	;if no carry, just do nothing
		
		LD A,H
		ADD A,8
		LD H,A				;if carry change address properly
		
				
TileBlock4
		POP BC				; restore block size in BC
		DEC C				; decrease number of tiles in collumn yet to do
		JR NZ, TileBlock1
		RET
		
;-----------------------------------------		
AttrBlock
;fill rectangular area with attribute
	;BC: area size B-width, C-height
	;DE:area left upper corner
	;A: attribute	
	
		PUSH BC		;check if ink set to black, if so do not change ink on screen
		LD B,A
		AND 7
		LD A,B
		POP BC
		JR Z,AttrBlock4
		
;draw ink option				
AttrBlock2
		PUSH BC
		PUSH DE
AttrBlock3
		LD (DE),A
		INC DE 		;go to next square in the row
		DJNZ AttrBlock3
		
		POP DE
		POP BC
		PUSH HL ;next line of attributes
		LD HL,32 ;(DE=DE+32)
		ADD HL,DE 
		EX DE,HL
		POP HL
	
		DEC C ;decreases number of lines yet to do 
		
		JR NZ,AttrBlock2
		RET
		
;no draw ink option
AttrBlock4
		PUSH AF
		PUSH BC
		PUSH DE
		LD C,A;stores attribute IN C
AttrBlock5	
		LD A,(DE)
		AND 7;extract ink
		ADD A,C;combine with attribute
		LD (DE),A
		INC DE ;goes to next square IN the row
		DJNZ AttrBlock5
		POP DE
		POP BC
		
		PUSH HL ;next line of attributes
		LD HL,32 ;(DE=DE+32)
		ADD HL,DE 
		EX DE,HL
		POP HL
		POP AF;restore attribute
		
		DEC C ;decreases number of lines yet to do 
		
		JR NZ,AttrBlock4
		RET

;--------------------------------------
;draw a single tile  on the screen
	;HL: upper left tile corner size	
	;BC: tile size (not a parameter but read by this routine form tile data)
	;DE: tile memory address	

;data are stored as follows (example shows tile 3 chars wide AND 2 chars high- 6*8=48 bytes):
;	1	9	17		
;	2	10	18
;	..	..	..
;	8	16	24
		
;	25	33	41
;	26	34	42
;	..	..	..
;	32	40	48
		
;Sevenup output options: attributes at the end, byte sort priority: char line, X char, Y char	

; empty characterss (chars with all 8 bytes equal to 0) are coded with just one byte to save space
; to code such an empty square number 16 is used
;FIRST BYTE OF 8 BYTES IN A CHAR CANNOT BE 16, BECAUSE PROGRAM WILL INTERPRET IT AS AN EMPTY SQUARE!!!
		
DrawTile

	LD A,(DE)		; load tile size to BC
	LD B,A 			; x width
	INC DE
	LD A,(DE)
	LD C,A 			; y-height	
	INC DE			
				
	PUSH HL			;store position
	PUSH BC			;store size
						
		

DrawTile1	
			PUSH HL
			PUSH BC
		
;this loop4 draws one line of chars
DrawTile2					
					PUSH HL
					LD A, (DE)
					CP 16	  			; number 16 codes empty character, when it is detected program jumps to another character
						  	  			; this routine saves data space	
					JR NZ, DrawTile3 	; when first byte is not 16, go to character drawing
					
					INC DE 				; when first byte is 16 it is an empty character,so skip it  go to next character 
					JR DrawTile4  
					
DrawTile3		
;draw a character
				LD A, (DE)
					LD (HL),A ;repeated 8 times, quicker than A loop
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC H
					INC DE
				LD A, (DE)
					LD (HL),A
					INC DE
						
DrawTile4		
			LD A,L
			POP HL 				;restore H
			LD L,A
			INC L				;moves one character right 
			 
			DJNZ DrawTile2 
			POP BC
			POP HL		
; go to next line of tile characters
			LD A,L
			ADD A,32
			LD L,A
			JR NC,DrawTile5		;not carry=no going to next segment, so no H changes needed
				
;change H only if going to next screen segment	
			LD A,H
			ADD A,8
			LD H,A
					
DrawTile5			
			DEC C 				;decrease number of character lines left to do
			JR NZ, DrawTile1
		
; graphics are ready, now paint attributes
; DE is already increased to next byte where attributes data begin
						
		POP BC
		POP HL		
							
;ATTRIBUTES
	;BC: tile size (B-width C-height)
	;DE:start of attributes data		
	;HL: screen address of left upper tile corner				
TileAttrs		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A ;transform graphics address to attribute address
			
TileAttr1
	 	PUSH BC
		PUSH HL
		
;draw one row of attributes
TileAttr2
	    LD A,(DE)
		OR A				;check if attribute is set to 0
		JR Z, TileAttr3		; if so, don't paint attribute and leave the square as it is
		LD (HL),A			; paint attribute
TileAttr3
		INC HL 				;go to next square in the row
		INC DE
		DJNZ TileAttr2
	
		POP HL
		PUSH DE  
		LD DE,32 
		ADD HL,DE			;go to next row of attributes
		POP DE 	 
		POP BC
		DEC C 				;decrease number of rows left to do 
		JR NZ, TileAttr1
		RET					;end of tile drawing


;-----------------------------------

MirrorArea
;make mirror of graphic area in the screen
	;HL: upper left corner of mirrored area
	;BC: area size
	;DE: doesn't contain any parameters but is used
	
		
;The routine has to match to every character in the area to the opposite mirror character
;it has to count it address AND place it IN DE using initial char address placed IN HL
; vertical coordinate is easy, it is just D=H
; there is more trouble with counting E
; we should use the equation:
; E= 2*l0 + B -1 -L
; where l0 - horizontal coordinate of leftmost char IN row 
; B - horizontal size of the area (stored IN B
; but if we put L into equation as l0 we have its simpler version:
; E=2*L+B-1-L=L+B-1
		

	PUSH HL
	PUSH BC ;stores	mirror block data for further use IN attribute mirror routine	
		
		
MirrorArea1
		PUSH HL
		PUSH BC		
			

;counts E					
			LD A,L		; A=l0
			ADD A,B		; A=l0+B
			DEC A			; A=l0+B-1
			LD E,A		; E=l0+B-1
			LD D,H
	
;if B is even (E.g. 8)the loop should be egzecuted B/2 times(4)
;if B is odd  (E.g. 11)the loop should be egzecuted B/2+1 times(6)

			
			XOR A;A=0
			SRL B
			ADC A,B
			LD B,A	
		
;mirrors one line of characters
MirrorArea2
		PUSH BC
		PUSH HL
		PUSH DE
			
		LD C,8
		

;mirrors 2 characters
MirrorArea3
				LD A,(HL)	
				LD B,0
				RLCA
				RR B
				RLCA
				RR B
				RLCA 
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B
				RLCA
				RR B		;byte with address in HL is mirrored and stored in B
				
				LD A,(DE)
				PUSH BC
				LD C,0
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				RLCA
				RR C
				LD A,C
				POP BC		;byte with address in DE is mirrored and stored in A
				
				
				LD (HL),A
				LD A,B
				LD (DE),A 	;mirrored HL and DE bytes are swapped
				INC H
				INC D
				
				DEC C
				JR NZ, MirrorArea3
	
		POP DE
		POP HL
		INC L
		DEC E		; goes to another pair of chars
		
		POP BC
		DJNZ MirrorArea2
		
	
	POP BC
	POP HL	
	
;goes to another line of chars	
	LD A,L
	ADD A,32	
	LD L,A		
	
;checks if mirrored line goes to next screen segment
	JR NC, MirrorArea4 		;if not=no carry, do nothing
	LD A,H
	ADD A,8
	LD H,A 					; if so, change address properly
		
MirrorArea4
		DEC C				; decreases number of lines yet to do			
		JR NZ, MirrorArea1	; begin routine from the start for another line

;Now it its time to mirror attributes
	
		POP BC
		POP HL
		
		LD A,80
		SRL H
		SRL H
		SRL H
		ADD A,H
		LD H,A ; counts address of first attribute char

		;counts address for opposite char (to HL char)IN the line
		;AND places it IN DE				
		
MrAttr1	PUSH BC
		PUSH HL
		
		LD A,L		; A=l0
		ADD A,B		; A=l0+B
		DEC A			; A=l0+B-1
		LD E,A		; E=l0+B-1
		LD D,H		;HL is mirrored IN DE now
		
		SRL B ;B is Divided by two
			;Mirrored block is B chars wide
			;but only B/2 chars attributes swaps are necessary
			; IN case B is even (E.g. 6) all attributes are symetrically swapped
			;IN case B is odd (E.g. 11) all are swapped except for the middle char
			;which doesnt need attribute change
		
MrAttr2	LD A,(HL)
		LD C,A	;HL attribute is placed IN C which is free
		LD A,(DE)
		LD (HL),A	;(DE) is placed IN (HL)
		LD A,C
		LD (DE),A	;(HL) is placed IN (DE)
		
		INC L
		DEC E	;go to another par of attributes
		 
		DJNZ MrAttr2 ; decreases B AND repeats routine until the whole line is swapped
		
		POP HL
		POP BC
		
		PUSH DE
		LD DE,32
		ADD HL,DE;go to next line
		POP DE	
		
		DEC C ; number of lines yet to do
		JR NZ, MrAttr1; repeat the whole routine for the new line
		
		RET

;--------------------
DrawMenu
;draw menu in upper part of screen
	;clear menu area
	XOR A
	LD HL,16384
	LD (HL),A
	LD DE, 16385
	LD BC, 2047
	LDIR	
		
;draw menu parts	
	LD BC,8193;B=32,C=1
	LD DE,Border01
	LD HL,16544
	CALL TileBlock	
	
	;energy border1
	LD HL, 16416
	LD DE,Border02
	PUSH DE
	CALL DrawTile	
	POP DE
	
	;% done border 1
	LD HL,16440
	CALL DrawTile
	
	;energy border 2
	LD HL,16417
	LD BC,1537;B=6,C=1
	
	PUSH BC
	PUSH DE
	LD DE,Border03
	CALL TileBlock	
		
	;%done border2
	LD HL,16441
	POP DE
	POP BC
	PUSH BC
	CALL TileBlock
	
	POP BC
	LD A,2	
	LD DE,22617
	CALL AttrBlock
		
	LD DE,Border04
	LD B,14
	LD C,1
	LD HL,16425
	CALL TileBlock
	
;menu texts			
	LD HL,16384
	LD DE,MenuText
	CALL TypeText
		
;draw score bar
	LD HL, 16473		;screen position
	LD A,(GameScore)	;score = (0-96)
	CALL DrawBar
	
;draw energy bar
	LD HL, 16449		;screen position
	LD A,(PlayerEnergy)	;energy = (0-96)
	CALL DrawBar
		

;--------------	
ItemDescribtions	
;First item
	LD A,(Item1Held)			
	LD HL,16457				;item nr 1 text screen position		
	CALL TypeItemDescribtion
;Second item
	LD A,(Item2Held)
	LD HL,16489				;item nr 2 text screen position	
	CALL TypeItemDescribtion
	
ColorizeDescribtions
;highlight selected item using attributes
	LD A,(SelectedItem)
	OR A
						
;colorize first line	
	PUSH AF
	LD A,7
	JR NZ,FirstItemLine
	LD A,78
FirstItemLine 
	LD HL,22601
	CALL HighlightMenuLine
	
;colorize second line	
	POP AF
	LD A,7
	JR Z,SecondItemLine
	LD A,78	
SecondItemLine	
	LD HL, 22633
	CALL HighlightMenuLine
	
	RET	;end of menu drawing

;-------------	
HighlightMenuLine
;fill one menu line with attribute (A=attribute,HL=screen attribute address) 		
	LD (HL),A
	LD D,H
	LD E,L
	INC DE		;DE=HL+1	
	LD BC,13
	LDIR					
	RET			

;-------------		
TypeItemDescribtion
;A: item number

;clear previous describtion
	PUSH HL
	PUSH AF	
	LD DE,ClearItemText
	CALL TypeText
	POP AF
	POP HL

;check if item is held or not	
	OR A	
	RET Z 					;no item -> don't type any describtion and return	

;count item describtion address
	PUSH HL
	LD HL,GameItemData-8
	LD B,A
	LD DE,8 				;one item data takes 8 bytes
FindItemDescrib
	ADD HL,DE				;go to next item	
	DJNZ FindItemDescrib	;repeat until item nr N is reached	and its address placed in HL	
	LD DE,6
	ADD HL,DE				;HL contains now adress	where text adress is placed

;type new describtion
	LD E,(HL)
	INC HL
	LD D,(HL)				;DE loaded with text address
	POP HL					;HL loaded with text screen position		
	CALL TypeText
	RET			

;-----------------		
DrawBar
;draw bar, which shows precent of completed game or player energy
;HL: bar position
;A: bar value

	SRA A 				;A=A/2 because we display walue 0-96 with 48 pixel wide bar
	LD D,A

DrawBar2	
	CP 8				;if A<=8, it means we reached last byte of bar, which isn't full
	JR C,LastBarByte
	JR Z,LastBarByte
	
	PUSH AF
	LD A,255
	LD B,8
	PUSH HL		
DrawBar3
;draw  full byte
	LD (HL),A
	INC H		
	DJNZ DrawBar3
	POP HL
	INC L				;go to next character
	POP AF
	SUB 8				;change bar width left to be drawn
	JR DrawBar2				
	
LastBarByte
;count width				
	OR A
	RET Z				;return if no need to draw last byte
	
	LD B,A
	XOR A
	
LastBarByte2
	SCF 				;carry=1
	RRA 				;create in A e.g. 11100000b
	DJNZ LastBarByte2
			
	LD B,8				;draw last ,not completely full score char
LastBarByte3
	LD (HL),A		
	INC H	
	DJNZ LastBarByte3
	
;clear next byte after bar (important when energy decreases
;not to leave rubbish pixels)	
	LD A,D
	CP 40				;if bar is almost 100% don't clear next byte
	RET NC				;as it would clear menu graphics
	
	INC L				;go to next byte
	LD A,H
	SUB 8
	LD H,A
	
	XOR A
	LD B,8
LastBarByte4
	LD (HL),A
	INC H
	DJNZ LastBarByte4	
	
	RET		
				
;----------------------------------------
;refresh current room (draw again current room after item action)	
RefreshCurrentRoom		
	LD HL,(CurrentRoom)
	CALL DrawRoom

;draw player and store new background, forgetting about background from room left
	LD HL,(OldPlayerPos)	
	CALL DrawPlayer	
	RET
;----------------		
GoIntoNewRoom
;DE address of room data
;BC position of player in new room
	LD A,20
GoIntoNewRoom2
	HALT
	DEC A
	JR NZ,GoIntoNewRoom2
	
;draw new room
	EX DE,HL; HL loaded with room address
	PUSH BC
	CALL DrawRoom
;draw player and store new background, forgetting about background from room left
	
	;update position
	POP BC
	LD (OldPlayerPos),BC
	LD (NewPlayerPos),BC
	
;store background and draw player	
	LD H,B
	LD L,C ;HL loaded with player position
	
	CALL DrawPlayer
	RET

